@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2020 Xorg
+*    Copyright © 2014-2021 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@
 #include <math.h>
 #include <pthread.h>
 #include <locale.h>
-#include <libintl.h>
 #include <sys/utsname.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -43,6 +42,10 @@
 # include <sys/sysctl.h>
 #endif
 
+#if HAS_GETTEXT
+# include <libintl.h>
+#endif
+
 #if HAS_LIBCPUID
 # include <libcpuid/libcpuid.h>
 # include "databases.h"
@@ -226,7 +229,7 @@ static int cpu_technology(Labels *data)
 		}
 	}
 
-	MSG_WARNING(_("Your CPU does not belong in database ==> %s, model: %i, ext. model: %i, ext. family: %i"),
+	MSG_WARNING(_("Your CPU is not present in the database ==> %s, model: %i, ext. model: %i, ext. family: %i"),
 	            data->tab_cpu[VALUE][SPECIFICATION], l_data->cpu_model, l_data->cpu_ext_model, l_data->cpu_ext_family);
 	RETURN_OR_EXIT(2);
 }
@@ -272,10 +275,10 @@ static int call_libcpuid_static(Labels *data)
 	/* Basically fill CPU tab */
 	casprintf(&data->tab_cpu[VALUE][CODENAME],      false, "%s",  datanr.cpu_codename);
 	casprintf(&data->tab_cpu[VALUE][SPECIFICATION], false, "%s",  datanr.brand_str);
-	casprintf(&data->tab_cpu[VALUE][FAMILY],        false, "%#X", datanr.family);
-	casprintf(&data->tab_cpu[VALUE][EXTFAMILY],     false, "%#X", datanr.ext_family);
-	casprintf(&data->tab_cpu[VALUE][MODEL],         false, "%#X", datanr.model);
-	casprintf(&data->tab_cpu[VALUE][EXTMODEL],      false, "%#X", datanr.ext_model);
+	casprintf(&data->tab_cpu[VALUE][FAMILY],        false, "0x%X", datanr.family);
+	casprintf(&data->tab_cpu[VALUE][EXTFAMILY],     false, "0x%X", datanr.ext_family);
+	casprintf(&data->tab_cpu[VALUE][MODEL],         false, "0x%X", datanr.model);
+	casprintf(&data->tab_cpu[VALUE][EXTMODEL],      false, "0x%X", datanr.ext_model);
 	casprintf(&data->tab_cpu[VALUE][STEPPING],      false, "%d",  datanr.stepping);
 	casprintf(&data->tab_cpu[VALUE][CORES],         true,  "%d",  datanr.num_cores);
 	casprintf(&data->tab_cpu[VALUE][THREADS],       true,  "%d",  datanr.num_logical_cpus);
@@ -817,7 +820,7 @@ static int find_devices(Labels *data)
 				case 0x8086: gpu_vendor = "Intel";  break;
 				case 0x10DE: gpu_vendor = "NVIDIA"; break;
 				default:     gpu_vendor = DEVICE_VENDOR_STR(dev);
-				             MSG_WARNING(_("Your GPU vendor is unknown: %s (%#X)"), gpu_vendor, dev->vendor_id);
+				             MSG_WARNING(_("Your GPU vendor is unknown: %s (0x%X)"), gpu_vendor, dev->vendor_id);
 			}
 
 			memset(gpu_driver, 0, MAXSTR);
@@ -861,6 +864,7 @@ static bool can_access_sys_debug_dri(Labels *data)
 			ret = -2;
 	}
 
+	MSG_DEBUG("can_access_sys_debug_dri() ==> %i", ret);
 	return !ret;
 }
 #endif /* __linux__ */
@@ -870,10 +874,10 @@ static int gpu_monitoring(Labels *data)
 {
 #ifdef __linux__
 	bool gpu_ok;
-	int ret_drm, ret_hwmon, ret_temp, ret_load, ret_gclk, ret_mclk;
+	int ret_drm, ret_hwmon, ret_temp, ret_load, ret_gclk, ret_mclk, ret_vram_used, ret_vram_total, ret_gvolt, ret_gpwr;
 	uint8_t i, card_number, failed_count = 0, fglrx_count = 0, nvidia_count = 0;
-	double divisor;
-	char *temp = NULL, *gclk = NULL, *mclk = NULL, *load = NULL;
+	long double divisor_temp, divisor_gclk, divisor_mclk, divisor_vram, divisor_gvolt, divisor_gpwr;
+	char *temp = NULL, *gclk = NULL, *mclk = NULL, *load = NULL, *vram_used = NULL, *vram_total = NULL, *gvolt = NULL, *gpwr = NULL;
 	static bool once_error = true;
 	static char *cached_paths_drm[LASTGRAPHICS / GPUFIELDS] = { NULL };
 	static char *cached_paths_hwmon[LASTGRAPHICS / GPUFIELDS] = { NULL };
@@ -881,14 +885,23 @@ static int gpu_monitoring(Labels *data)
 	MSG_VERBOSE("%s", _("Retrieving GPU clocks"));
 	for(i = 0; i < data->gpu_count; i++)
 	{
-		gpu_ok    = gpu_is_on(data->g_data->device_path[i]);
-		ret_drm   = 0;
-		ret_hwmon = 0;
-		ret_temp  = -1;
-		ret_load  = -1;
-		ret_gclk  = -1;
-		ret_mclk  = -1;
-		divisor   = 1.0;
+		gpu_ok         = gpu_is_on(data->g_data->device_path[i]);
+		ret_drm        = 0;
+		ret_hwmon      = 0;
+		ret_temp       = -1;
+		ret_load       = -1;
+		ret_gclk       = -1;
+		ret_mclk       = -1;
+		ret_vram_used  = -1;
+		ret_vram_total = -1;
+		ret_gvolt      = -1;
+		ret_gpwr       = -1;
+		divisor_temp   = 1.0;
+		divisor_gclk   = 1.0;
+		divisor_mclk   = 1.0;
+		divisor_vram   = 1.0;
+		divisor_gvolt  = 1.0;
+		divisor_gpwr   = 1.0;
 
 		if(gpu_ok && (data->g_data->gpu_driver[i] == GPUDRV_UNKNOWN))
 		{
@@ -902,8 +915,11 @@ static int gpu_monitoring(Labels *data)
 			casprintf(&data->tab_graphics[VALUE][GPU1DRIVER      + i * GPUFIELDS], false, _("None"));
 			casprintf(&data->tab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS], false, "---");
 			casprintf(&data->tab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS], false, "---");
+			casprintf(&data->tab_graphics[VALUE][GPU1MEMUSED     + i * GPUFIELDS], false, "---");
 			casprintf(&data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS], false, "---");
 			casprintf(&data->tab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS], false, "---");
+			casprintf(&data->tab_graphics[VALUE][GPU1VOLTAGE     + i * GPUFIELDS], false, "---");
+			casprintf(&data->tab_graphics[VALUE][GPU1POWERAVG    + i * GPUFIELDS], false, "---");
 			continue;
 		}
 
@@ -915,11 +931,11 @@ static int gpu_monitoring(Labels *data)
 			case GPUDRV_NOUVEAU:
 			case GPUDRV_NOUVEAU_BUMBLEBEE:
 				/* HWmon */
-				divisor = 1000.0;
+				divisor_temp = 1e3;
 				if((cached_paths_hwmon[i] == NULL) && (data->g_data->gpu_driver[i] != GPUDRV_INTEL))
-					ret_hwmon = request_sensor_path(format("%s/hwmon", data->g_data->device_path[i]), &cached_paths_hwmon[i], RQT_GPU_TEMPERATURE);
+					ret_hwmon = request_sensor_path(format("%s/hwmon", data->g_data->device_path[i]), &cached_paths_hwmon[i], RQT_GPU_HWMON);
 				if(!ret_hwmon && (cached_paths_hwmon[i] != NULL))
-					ret_temp = fopen_to_str(&temp, cached_paths_hwmon[i]);
+					ret_temp = fopen_to_str(&temp, "%s/temp1_input", cached_paths_hwmon[i]);
 
 				/* DRM */
 				if(cached_paths_drm[i] == NULL)
@@ -935,14 +951,24 @@ static int gpu_monitoring(Labels *data)
 		{
 			case GPUDRV_AMDGPU:
 			{
-				const char *amdgpu_gpu_busy_file = format( "%s/device/gpu_busy_percent", cached_paths_drm[i]);
+				const char *amdgpu_gpu_busy_file = format("%s/device/gpu_busy_percent", cached_paths_drm[i]);
+				MSG_DEBUG("gpu_monitoring: amdgpu: amdgpu_gpu_busy_file=%s", amdgpu_gpu_busy_file);
 				// ret_temp obtained above
 				if(!access(amdgpu_gpu_busy_file, F_OK)) // Linux 4.19+
 					ret_load = fopen_to_str(&load, "%s", amdgpu_gpu_busy_file);
 				else if(can_access_sys_debug_dri(data))
 					ret_load = popen_to_str(&load, "awk '/GPU Load/ { print $3 }' %s/%u/amdgpu_pm_info", SYS_DEBUG_DRI, card_number);
-				ret_gclk  = popen_to_str(&gclk, "awk -F '(: |Mhz)' '/\\*/ { print $2 }' %s/device/pp_dpm_sclk", cached_paths_drm[i]);
-				ret_mclk  = popen_to_str(&mclk, "awk -F '(: |Mhz)' '/\\*/ { print $2 }' %s/device/pp_dpm_mclk", cached_paths_drm[i]);
+				ret_gclk       = fopen_to_str(&gclk,       "%s/freq1_input",                 cached_paths_hwmon[i]);
+				ret_mclk       = fopen_to_str(&mclk,       "%s/freq2_input",                 cached_paths_hwmon[i]);
+				ret_vram_used  = fopen_to_str(&vram_used,  "%s/device/mem_info_vram_used",   cached_paths_drm[i]);
+				ret_vram_total = fopen_to_str(&vram_total, "%s/device/mem_info_vram_total",  cached_paths_drm[i]);
+				ret_gvolt      = fopen_to_str(&gvolt,      "%s/in0_input",                   cached_paths_hwmon[i]);
+				ret_gpwr       = fopen_to_str(&gpwr,       "%s/power1_average",              cached_paths_hwmon[i]);
+				divisor_gclk  = 1e6;
+				divisor_mclk  = 1e6;
+				divisor_vram  = 1 << 20;
+				divisor_gvolt = 1e3;
+				divisor_gpwr  = 1e6;
 				break;
 			}
 			case GPUDRV_FGLRX:
@@ -950,21 +976,33 @@ static int gpu_monitoring(Labels *data)
 				ret_load  = popen_to_str(&load, "aticonfig --adapter=%1u --odgc | awk '/GPU load/ { sub(\"%\",\"\",$4); print $4 }'", fglrx_count);
 				ret_gclk  = popen_to_str(&gclk, "aticonfig --adapter=%1u --odgc | awk '/Current Clocks/ { print $4 }'",               fglrx_count);
 				ret_mclk  = popen_to_str(&mclk, "aticonfig --adapter=%1u --odgc | awk '/Current Clocks/ { print $5 }'",               fglrx_count);
+				ret_vram_used  = -1;
+				ret_vram_total = -1;
+				ret_gvolt      = -1;
+				ret_gpwr       = -1;
 				fglrx_count++;
 				break;
 			case GPUDRV_INTEL:
-				ret_temp  = -1;
-				ret_load  = -1;
+				ret_temp       = -1;
+				ret_load       = -1;
 				ret_gclk  = fopen_to_str(&gclk, "%s/gt_cur_freq_mhz", cached_paths_drm[i]);
-				ret_mclk  = -1;
+				ret_mclk       = -1;
+				ret_vram_used  = -1;
+				ret_vram_total = -1;
+				ret_gvolt      = -1;
+				ret_gpwr       = -1;
 				break;
 			case GPUDRV_RADEON:
 				// ret_temp obtained above
-				ret_load  = -1;
+				ret_load       = -1;
 				ret_gclk  = can_access_sys_debug_dri(data) ? popen_to_str(&gclk, "awk -F '(sclk: | mclk:)' 'NR==2 { print $2 }' %s/%u/radeon_pm_info", SYS_DEBUG_DRI, card_number) : -1;
 				ret_mclk  = can_access_sys_debug_dri(data) ? popen_to_str(&mclk, "awk -F '(mclk: | vddc:)' 'NR==2 { print $2 }' %s/%u/radeon_pm_info", SYS_DEBUG_DRI, card_number) : -1;
-				if((gclk != NULL) && (strlen(gclk) >= 2)) gclk[strlen(gclk) - 2] = '\0';
-				if((mclk != NULL) && (strlen(mclk) >= 2)) mclk[strlen(mclk) - 2] = '\0';
+				ret_vram_used  = -1;
+				ret_vram_total = -1;
+				ret_gvolt      = -1;
+				ret_gpwr       = -1;
+				divisor_gclk = 100.0;
+				divisor_mclk = 100.0;
 				break;
 			case GPUDRV_NVIDIA:
 			case GPUDRV_NVIDIA_BUMBLEBEE:
@@ -972,10 +1010,15 @@ static int gpu_monitoring(Labels *data)
 				/* Doc: https://nvidia.custhelp.com/app/answers/detail/a_id/3751/~/useful-nvidia-smi-queries */
 				const char *nvidia_cmd_base = (data->g_data->gpu_driver[i] == GPUDRV_NVIDIA_BUMBLEBEE) ? "optirun -b none nvidia-smi -c :8" : "nvidia-smi";
 				const char *nvidia_cmd_args = format("%s --format=csv,noheader,nounits --id=%1u", nvidia_cmd_base, nvidia_count);
+				MSG_DEBUG("gpu_monitoring: nvidia: nvidia_cmd_args=%s", nvidia_cmd_args);
 				ret_temp  = popen_to_str(&temp, "%s --query-gpu=temperature.gpu", nvidia_cmd_args);
 				ret_load  = popen_to_str(&load, "%s --query-gpu=utilization.gpu", nvidia_cmd_args);
 				ret_gclk  = popen_to_str(&gclk, "%s --query-gpu=clocks.gr",       nvidia_cmd_args);
 				ret_mclk  = popen_to_str(&mclk, "%s --query-gpu=clocks.mem",      nvidia_cmd_args);
+				ret_vram_used  = -1;
+				ret_vram_total = -1;
+				ret_gvolt      = -1;
+				ret_gpwr       = -1;
 				nvidia_count++;
 				break;
 			}
@@ -984,10 +1027,15 @@ static int gpu_monitoring(Labels *data)
 			{
 				char *pstate = NULL;
 				int ret_pstate = popen_to_str(&pstate, "grep '*' %1$s/%2$u/pstate || sed -n 1p %1$s/%2$u/pstate ", SYS_DEBUG_DRI, card_number);
+				MSG_DEBUG("gpu_monitoring: nouveau: pstate=%s", pstate);
 				// ret_temp obtained above
 				ret_load  = -1;
 				ret_gclk  = !ret_pstate && can_access_sys_debug_dri(data) ? popen_to_str(&gclk, "echo %s | grep -oP '(?<=core )[^ ]*' | cut -d- -f2", pstate) : -1;
 				ret_mclk  = !ret_pstate && can_access_sys_debug_dri(data) ? popen_to_str(&mclk, "echo %s | grep -oP '(?<=memory )[^ ]*'",             pstate) : -1;
+				ret_vram_used  = -1;
+				ret_vram_total = -1;
+				ret_gvolt      = -1;
+				ret_gpwr       = -1;
 				FREE(pstate);
 				break;
 			}
@@ -998,22 +1046,34 @@ static int gpu_monitoring(Labels *data)
 		}
 
 		if(!ret_load)
-			casprintf(&data->tab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS], false, "%s%%",  load);
+			casprintf(&data->tab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS], false, "%s%%", load);
 		if(!ret_gclk)
-			casprintf(&data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS], true, "%s MHz", gclk);
+			casprintf(&data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS], true, "%.2Lf MHz", strtoull(gclk, NULL, 10) / divisor_gclk);
 		if(!ret_mclk)
-			casprintf(&data->tab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS], true, "%s MHz", mclk);
+			casprintf(&data->tab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS], true, "%.2Lf MHz", strtoull(mclk, NULL, 10) / divisor_mclk);
+		if(!ret_vram_used && !ret_vram_total)
+			casprintf(&data->tab_graphics[VALUE][GPU1MEMUSED     + i * GPUFIELDS], true, "%.0Lf %s / %.0Lf %s",
+				strtoull(vram_used,  NULL, 10) / divisor_vram, UNIT_MIB,
+				strtoull(vram_total, NULL, 10) / divisor_vram, UNIT_MIB);
+		if(!ret_gvolt)
+			casprintf(&data->tab_graphics[VALUE][GPU1VOLTAGE     + i * GPUFIELDS], true, "%.2Lf V", strtoull(gvolt, NULL, 10) / divisor_gvolt);
+		if(!ret_gpwr)
+			casprintf(&data->tab_graphics[VALUE][GPU1POWERAVG    + i * GPUFIELDS], true, "%.2Lf W", strtoull(gpwr, NULL, 10) / divisor_gpwr);
 skip_clocks:
 		if(!ret_temp)
-			casprintf(&data->tab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS], true, "%.2f°C", atof(temp) / divisor);
+			casprintf(&data->tab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS], true, "%.2Lf°C", strtoull(temp, NULL, 10) / divisor_temp);
 
-		if(ret_temp && ret_load && ret_gclk && ret_mclk)
+		if(ret_temp && ret_load && ret_gclk && ret_mclk && ret_vram_used && ret_vram_total && ret_gvolt && ret_gpwr)
 			failed_count++;
 
 		FREE(temp);
 		FREE(load);
 		FREE(gclk);
 		FREE(mclk);
+		FREE(vram_used);
+		FREE(vram_total);
+		FREE(gvolt);
+		FREE(gpwr);
 	}
 
 	if(once_error && failed_count)
@@ -1077,7 +1137,7 @@ static int system_dynamic(Labels *data)
 	MemoryData *m_data = data->m_data;
 
 #if HAS_LIBPROCPS
-	const int div = 1e3;
+	const int div = 1 << 10;
 
 	MSG_VERBOSE("%s", _("Calling libprocps"));
 	/* System uptime */
@@ -1096,7 +1156,7 @@ static int system_dynamic(Labels *data)
 
 #if HAS_LIBSTATGRAB
 	static bool called = false;
-	const int div = 1e6;
+	const int div = 1 << 20;
 	sg_mem_stats *mem; /* Memory labels */
 	sg_swap_stats *swap;
 	sg_host_info *info;
@@ -1126,8 +1186,8 @@ static int system_dynamic(Labels *data)
 #endif /* HAS_LIBSTATGRAB */
 	/* Memory labels */
 	for(i = USED; i < SWAP; i++)
-		casprintf(&data->tab_system[VALUE][i], false, "%5u %s / %5u %s", m_data->mem_usage[j++], UNIT_MB, m_data->mem_total, UNIT_MB);
-	casprintf(&data->tab_system[VALUE][SWAP], false, "%5u %s / %5u %s", m_data->mem_usage[j], UNIT_MB, m_data->swap_total, UNIT_MB);
+		casprintf(&data->tab_system[VALUE][i], false, "%5u %s / %5u %s", m_data->mem_usage[j++], UNIT_MIB, m_data->mem_total, UNIT_MIB);
+	casprintf(&data->tab_system[VALUE][SWAP], false, "%5u %s / %5u %s", m_data->mem_usage[j], UNIT_MIB, m_data->swap_total, UNIT_MIB);
 
 	/* Uptime label */
 	tm = gmtime(&uptime_s);
@@ -1279,17 +1339,21 @@ static int cputab_package_fallback(Labels *data)
 	else
 		db = package_unknown;
 
-	while(db[++i].codename != NULL)
+	while(db[++i].socket != NULL)
 	{
-		if((strstr(data->tab_cpu[VALUE][CODENAME], db[i].codename) != NULL) &&
-		  ((db[i].model == NULL) || (strstr(data->tab_cpu[VALUE][SPECIFICATION], db[i].model) != NULL)))
+		const bool codename_defined = (db[i].codename != NULL);
+		const bool model_defined    = (db[i].model    != NULL);
+		const bool codename_matchs  = codename_defined && (strstr(data->tab_cpu[VALUE][CODENAME], db[i].codename)   != NULL);
+		const bool model_matchs     = model_defined    && (strstr(data->tab_cpu[VALUE][SPECIFICATION], db[i].model) != NULL);
+
+		if((codename_matchs && model_matchs) || (codename_matchs && !model_defined) || (!codename_defined && model_matchs))
 		{
 			casprintf(&data->tab_cpu[VALUE][PACKAGE], false, db[i].socket);
 			return 0;
 		}
 	}
 
-	MSG_WARNING(_("Your CPU socket does not belong in database ==> %s, codename: %s"),
+	MSG_WARNING(_("Your CPU socket is not present in the database ==> %s, codename: %s"),
 		    data->tab_cpu[VALUE][SPECIFICATION], data->tab_cpu[VALUE][CODENAME]);
 	data->tab_cpu[VALUE][PACKAGE][0] = '\0';
 	return 2;
@@ -1312,14 +1376,14 @@ static int cputab_multipliers_fallback(Labels *data)
 	/* Minimum multiplier */
 	if(!(err = fopen_to_str(&min_freq_str, "%s%i/cpufreq/cpuinfo_min_freq", SYS_CPU, opts->selected_core)))
 	{
-		data->cpu_min_mult = round((strtod(min_freq_str, NULL) / 1000) / data->bus_freq);
+		data->cpu_min_mult = round((strtod(min_freq_str, NULL) / 1e3) / data->bus_freq);
 		free(min_freq_str);
 	}
 
 	/* Maximum multiplier */
 	if(!(err = fopen_to_str(&max_freq_str, "%s%i/cpufreq/cpuinfo_max_freq", SYS_CPU, opts->selected_core)))
 	{
-		data->cpu_max_mult = round((strtod(max_freq_str, NULL) / 1000) / data->bus_freq);
+		data->cpu_max_mult = round((strtod(max_freq_str, NULL) / 1e3) / data->bus_freq);
 		free(max_freq_str);
 	}
 #endif /* __linux__ */
@@ -1427,7 +1491,7 @@ static int cputab_temp_fallback(Labels *data)
 	{
 		if(!(err = fopen_to_str(&temp, cached_paths[opts->selected_core])))
 		{
-			casprintf(&data->tab_cpu[VALUE][TEMPERATURE], true, "%.2f°C", atof(temp) / 1000.0);
+			casprintf(&data->tab_cpu[VALUE][TEMPERATURE], true, "%.2f°C", atof(temp) / 1e3);
 			free(temp);
 		}
 	}
@@ -1458,7 +1522,7 @@ static int cputab_volt_fallback(Labels *data)
 	{
 		if(!(err = fopen_to_str(&voltage, cached_path)))
 		{
-			casprintf(&data->tab_cpu[VALUE][VOLTAGE], true, "%.3f V", atof(voltage) / 1000.0);
+			casprintf(&data->tab_cpu[VALUE][VOLTAGE], true, "%.3f V", atof(voltage) / 1e3);
 			free(voltage);
 		}
 	}
@@ -1481,7 +1545,7 @@ static int cputab_freq_fallback(Labels *data)
 	MSG_VERBOSE("%s", _("Retrieving CPU frequency in fallback mode"));
 	if(!(err = fopen_to_str(&freq, "%s%i/cpufreq/scaling_cur_freq", SYS_CPU, opts->selected_core)))
 	{
-		data->cpu_freq = (int) round(strtod(freq, NULL) / 1000.0);
+		data->cpu_freq = (int) round(strtod(freq, NULL) / 1e3);
 		casprintf(&data->tab_cpu[VALUE][CORESPEED], true, "%d MHz", data->cpu_freq);
 		free(freq);
 	}
