@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2019 Xorg
+*    Copyright © 2014-2021 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -28,15 +28,14 @@
 #include <stdbool.h>
 #include <pthread.h>
 #include <errno.h>
-#define HAVE_STDINT_H         /* Skip conflicts with <libcpuid/libcpuid_types.h> */
+#include <libintl.h>
 
 /* Software definition */
 #define PRGNAME               "CPU-X"
+#define PRGNAME_LOW           "cpu-x"
 #define PRGAUTH               "X0rg"
-#define PRGURL                "https://X0rg.github.io/CPU-X"
-#define UPDURL                "https://api.github.com/repos/"PRGAUTH"/"PRGNAME"/releases/latest"
-#define DOWNLOAD              "https://github.com/"PRGAUTH"/"PRGNAME"/releases/latest"
-#define PRGCPRGHT             "Copyright © 2014-2019 Xorg"
+#define PRGURL                "https://"PRGAUTH".github.io/"PRGNAME
+#define PRGCPRGHT             "Copyright © 2014-2021 Xorg"
 #define PRGLCNS               "GNU GPL v3"
 #define LCNSURL               "https://www.gnu.org/licenses/gpl.txt"
 
@@ -46,19 +45,31 @@
 #define BOLD_GREEN            "\x1b[1;32m"
 #define BOLD_YELLOW           "\x1b[1;33m"
 #define BOLD_BLUE             "\x1b[1;34m"
+#define BOLD_MAGENTA          "\x1b[1;35m"
 
 /* Utilities macro */
 #define LOCATION              PRGNAME, (strrchr(__FILE__, '/') ? strrchr(__FILE__, '/') + 1 : __FILE__), __LINE__
 #define GOTO_ERROR(str)       { snprintf(error_str, MAXSTR, str); goto error; }
+#define ALLOC_CHECK(ptr)      if(ptr == NULL) { MSG_ERRNO("%s", _("FATAL ERROR: could not allocate memory")); exit(255); }
+#define FREE(ptr)             { free(ptr); ptr = NULL; }
+#define IS_ROOT               (getuid() == 0)
+#define PRGINFO(out)          fprintf(out, "%s %s%s (%s %s, %s %s, %s %s)\n", PRGNAME, PRGVER, GITREV, __DATE__, __TIME__, SYSTEM_NAME, SYSTEM_PROCESSOR, C_COMPILER_ID, __VERSION__)
+#define UNUSED(x)             (void)(x)
 
 /* Formatted messages definition */
+#define MSG_BUFF_LEN          128
 #define MSG_STDOUT(fmt, ...)  fprintf(stdout, colorized_msg(DEFAULT, "%s", fmt), ##__VA_ARGS__)
 #define MSG_STDERR(fmt, ...)  fprintf(stderr, colorized_msg(DEFAULT, "%s", fmt), ##__VA_ARGS__)
-#define MSG_VERBOSE(fmt, ...) opts->verbose ? fprintf(stdout, colorized_msg(BOLD_GREEN, "%s", fmt), ##__VA_ARGS__) : 0
+#define MSG_VERBOSE(fmt, ...) opts->verbose ? fprintf(stdout, colorized_msg(BOLD_GREEN,   "%s", fmt), ##__VA_ARGS__) : 0
+#define MSG_DEBUG(fmt, ...)   opts->debug   ? fprintf(stdout, colorized_msg(BOLD_MAGENTA, "%s", fmt), ##__VA_ARGS__) : 0
 #define MSG_WARNING(fmt, ...) fprintf(stdout, colorized_msg(BOLD_YELLOW, "%s", fmt), ##__VA_ARGS__)
 #define MSG_ERROR(fmt, ...)   fprintf(stderr, colorized_msg(BOLD_RED, "%s:%s:%i: %s",      LOCATION, fmt), ##__VA_ARGS__)
 #define MSG_ERRNO(fmt, ...)   fprintf(stderr, colorized_msg(BOLD_RED, "%s:%s:%i: %s (%s)", LOCATION, fmt, strerror(errno)), ##__VA_ARGS__)
-#define _(msg)                gettext(msg)
+#if HAS_GETTEXT
+# define _(msg)               gettext(msg)
+#else
+# define _(msg)               msg
+#endif /* HAS_GETTEXT */
 #define N_(msg)               msg
 
 /* Options definition */
@@ -75,15 +86,39 @@
 #define MAXSTR                80       /* Max string */
 #define CACHEFIELDS           2        /* Nb of fields by cache frame */
 #define RAMFIELDS             2        /* Nb of fields by bank */
-#define GPUFIELDS             6        /* Nb of fields by GPU frame */
+#define GPUFIELDS             10        /* Nb of fields by GPU frame */
 #define BENCHFIELDS           2        /* Nb of fields by bench frame */
 
+#define UNIT_B                _("bytes")
+/* SI unit prefixes */
+// TRANSLATORS: kilobyte (10³)
+#define UNIT_KB               _("kB")
+// TRANSLATORS: megabyte (10⁶)
+#define UNIT_MB               _("MB")
+// TRANSLATORS: gigabyte (10⁹)
+#define UNIT_GB               _("GB")
+// TRANSLATORS: terabyte (10¹²)
+#define UNIT_TB               _("TB")
+/* Binary unit prefixes  */
+// TRANSLATORS: kibibyte (2¹⁰)
+#define UNIT_KIB               _("KiB")
+// TRANSLATORS: mebibyte (2²⁰)
+#define UNIT_MIB               _("MiB")
+// TRANSLATORS: gibibyte (2³⁰)
+#define UNIT_GIB               _("GiB")
+// TRANSLATORS: tebibyte (2⁴⁰)
+#define UNIT_TIB               _("TiB")
+
 /* Linux-specific paths definition */
 #define SYS_DMI               "/sys/devices/virtual/dmi/id"
 #define SYS_CPU               "/sys/devices/system/cpu/cpu"
 #define SYS_DRM               "/sys/class/drm/card"
 #define SYS_HWMON             "/sys/class/hwmon"
-#define SYS_DRI               "/sys/kernel/debug/dri"
+#define SYS_DEBUG             "/sys/kernel/debug"
+#define SYS_DEBUG_DRI         SYS_DEBUG"/dri"
+
+/* FreeBSD-specific paths definition */
+#define DEV_PCI               "/dev/pci"
 
 
 enum EnTabNumber
@@ -153,18 +188,19 @@ enum EnTabSystemExtra
 
 enum EnTabGraphics
 {
-	GPU1VENDOR, GPU1MODEL, GPU1TEMPERATURE, GPU1USAGE, GPU1CORECLOCK, GPU1MEMCLOCK,
-	GPU2VENDOR, GPU2MODEL, GPU2TEMPERATURE, GPU2USAGE, GPU2CORECLOCK, GPU2MEMCLOCK,
-	GPU3VENDOR, GPU3MODEL, GPU3TEMPERATURE, GPU3USAGE, GPU3CORECLOCK, GPU3MEMCLOCK,
-	GPU4VENDOR, GPU4MODEL, GPU4TEMPERATURE, GPU4USAGE, GPU4CORECLOCK, GPU4MEMCLOCK,
+	GPU1VENDOR, GPU1DRIVER, GPU1MODEL, GPU1TEMPERATURE, GPU1USAGE, GPU1MEMUSED, GPU1CORECLOCK, GPU1MEMCLOCK, GPU1VOLTAGE, GPU1POWERAVG,
+	GPU2VENDOR, GPU2DRIVER, GPU2MODEL, GPU2TEMPERATURE, GPU2USAGE, GPU2MEMUSED, GPU2CORECLOCK, GPU2MEMCLOCK, GPU2VOLTAGE, GPU2POWERAVG,
+	GPU3VENDOR, GPU3DRIVER, GPU3MODEL, GPU3TEMPERATURE, GPU3USAGE, GPU3MEMUSED, GPU3CORECLOCK, GPU3MEMCLOCK, GPU3VOLTAGE, GPU3POWERAVG,
+	GPU4VENDOR, GPU4DRIVER, GPU4MODEL, GPU4TEMPERATURE, GPU4USAGE, GPU4MEMUSED, GPU4CORECLOCK, GPU4MEMCLOCK, GPU4VOLTAGE, GPU4POWERAVG,
 	LASTGRAPHICS
 };
 
 enum EnGpuDrv
 {
-	GPUDRV_FGLRX, GPUDRV_AMDGPU, GPUDRV_RADEON,             // AMD
-	GPUDRV_INTEL,                                           // Intel
-	GPUDRV_NVIDIA, GPUDRV_NVIDIA_BUMBLEBEE, GPUDRV_NOUVEAU, // NVIDIA
+	GPUDRV_FGLRX, GPUDRV_AMDGPU, GPUDRV_RADEON,                                       // AMD
+	GPUDRV_INTEL,                                                                     // Intel
+	GPUDRV_NVIDIA, GPUDRV_NVIDIA_BUMBLEBEE, GPUDRV_NOUVEAU, GPUDRV_NOUVEAU_BUMBLEBEE, // NVIDIA
+	GPUDRV_VFIO,
 	GPUDRV_UNKNOWN
 };
 
@@ -184,6 +220,15 @@ enum EnTabAbout
 	LASTABOUT
 };
 
+enum EnOptKeymap
+{
+	ARROWS,
+	EMACS,
+	INVERTED_T,
+	VIM,
+	LASTKEYMAP
+};
+
 typedef struct
 {
 	int8_t  cpu_vendor_id;
@@ -234,9 +279,10 @@ typedef struct
 	char *tab_bench[2][LASTBENCH];
 	char *tab_about[LASTABOUT];
 
-	int     cpu_freq;
+	int     cpu_freq, socket_fd;
 	uint8_t cpu_count, cache_count, dimm_count, gpu_count;
 	double  bus_freq, cpu_min_mult, cpu_max_mult;
+	bool    reload;
 
 	LibcpuidData  *l_data;
 	BandwidthData *w_data;
@@ -247,9 +293,10 @@ typedef struct
 
 typedef struct
 {
-	bool     color, verbose, issue, use_network, update, debug_database, freq_fallback;
+	bool     color, verbose, debug, issue, use_network, with_daemon, debug_database, freq_fallback;
 	uint8_t  selected_page, selected_core, bw_test;
 	uint16_t output_type, refr_time;
+	enum EnOptKeymap keymap;
 } Options;
 
 typedef struct
@@ -260,7 +307,6 @@ typedef struct
 } Arrays;
 
 extern Options *opts;
-extern char    *binary_name, *new_version[2];
 
 
 /***************************** Defined in main.c *****************************/
@@ -286,22 +332,28 @@ char *format(char *str, ...);
 /* Similar to format(), but string can be colorized */
 char *colorized_msg(const char *color, const char *str, ...);
 
-/* Check if a command exists */
-bool command_exists(char *command);
-
 /* Open a file and put its content in a variable ('str' accept printf-like format) */
 int fopen_to_str(char **buffer, char *str, ...);
 
 /* Run a command and put output in a variable ('str' accept printf-like format) */
 int popen_to_str(char **buffer, char *str, ...);
 
-/* Load a kernel module */
-bool load_module(char *module);
+/* Check if a command exists */
+bool command_exists(char *cmd);
+
+/* Load a kernel module (return 0 on success) */
+int load_module(char *module, int *fd);
 
 /* Get a filename located in a directory corresponding to given request */
-enum RequestSensor { RQT_CPU_TEMPERATURE, RQT_CPU_TEMPERATURE_OTHERS, RQT_CPU_VOLTAGE, RQT_GPU_TEMPERATURE, RQT_GPU_DRM };
+enum RequestSensor { RQT_CPU_TEMPERATURE, RQT_CPU_TEMPERATURE_OTHERS, RQT_CPU_VOLTAGE, RQT_GPU_TEMPERATURE, RQT_GPU_DRM, RQT_GPU_HWMON };
 int request_sensor_path(char *base_dir, char **cached_path, enum RequestSensor which);
 
+/* Start daemon in background */
+const char *start_daemon(bool graphical);
+
+/* Check if daemon is running */
+bool daemon_is_alive(void);
+
 
 /***************************** External headers *****************************/
 
@@ -311,6 +363,12 @@ int fill_labels(Labels *data);
 /* Refresh some labels */
 int do_refresh(Labels *data);
 
+/* Establish connection to daemon */
+int connect_to_daemon(Labels *data);
+
+/* Dmidecode main function */
+int dmidecode(int quiet, void *cpux_pdata);
+
 /* Call Dmidecode through CPU-X but do nothing else */
 int run_dmidecode(void);
 
@@ -320,6 +378,9 @@ int run_bandwidth(void);
 /* Perform a multithreaded benchmark (compute prime numbers) */
 void start_benchmarks(Labels *data);
 
+/* Load and apply settings from GSettings */
+void load_settings(void);
+
 /* Start CPU-X in GTK mode */
 void start_gui_gtk(int *argc, char **argv[], Labels *data);
 
