@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2020 Xorg
+*    Copyright © 2014-2021 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <libintl.h>
+#include <sys/wait.h>
 #include "cpu-x.h"
 #include "ipc.h"
 #include "gui_gtk.h"
@@ -99,8 +100,11 @@ void start_gui_gtk(int *argc, char **argv[], Labels *data)
 static gboolean grefresh(GThrd *refr)
 {
 	int i;
+	gboolean ret = true;
 	Labels    *(data) = refr->data;
 	GtkLabels *(glab) = refr->glab;
+	uint16_t new_refr_time = g_settings_get_uint(settings, "refresh-time");
+	static uint16_t old_refr_time = 0;
 
 	opts->selected_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(glab->notebook));
 	do_refresh(data);
@@ -128,8 +132,11 @@ static gboolean grefresh(GThrd *refr)
 			{
 				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1TEMPERATURE + i * GPUFIELDS]);
 				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1USAGE       + i * GPUFIELDS]);
+				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1MEMUSED     + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1MEMUSED     + i * GPUFIELDS]);
 				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1CORECLOCK   + i * GPUFIELDS]);
 				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1MEMCLOCK    + i * GPUFIELDS]);
+				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1VOLTAGE     + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1VOLTAGE     + i * GPUFIELDS]);
+				gtk_label_set_text(GTK_LABEL(glab->gtktab_graphics[VALUE][GPU1POWERAVG    + i * GPUFIELDS]), data->tab_graphics[VALUE][GPU1POWERAVG    + i * GPUFIELDS]);
 			}
 			break;
 		case NO_BENCH:
@@ -143,7 +150,11 @@ static gboolean grefresh(GThrd *refr)
 
 	/* Destroy current timeout when refresh time is updated
 	Note: modify_refresh_time() will create a new timeout */
-	return (opts->refr_time == g_settings_get_uint(settings, "refresh-time"));
+	if(old_refr_time == 0)
+		old_refr_time = new_refr_time;
+	ret = (old_refr_time == new_refr_time);
+	old_refr_time = new_refr_time;
+	return ret;
 }
 
 /* Create new timeout when old one is destroyed */
@@ -683,25 +694,20 @@ void fill_frame(GtkWidget *widget, cairo_t *cr, GThrd *refr)
 
 	switch(page) /* Set differents level bar color */
 	{
-		case BARUSED:
-			cairo_pattern_add_color_stop_rgba(pat, 0, 1.00, 1.00, 0.15, 1);
-			cairo_pattern_add_color_stop_rgba(pat, 1, 1.00, 0.75, 0.15, 1);
+		case BARUSED: // yellow
+			pat = is_dark_theme(glab) ? cairo_pattern_create_rgba(0.01, 0.02, 0.37, 1.00) : cairo_pattern_create_rgba(1.00, 0.92, 0.23, 1.00);
 			break;
-		case BARBUFFERS:
-			cairo_pattern_add_color_stop_rgba(pat, 0, 0.00, 0.30, 0.75, 1);
-			cairo_pattern_add_color_stop_rgba(pat, 1, 0.25, 0.55, 1.00, 1);
+		case BARBUFFERS: // blue
+			pat = is_dark_theme(glab) ? cairo_pattern_create_rgba(0.01, 0.24, 0.54, 1.00) : cairo_pattern_create_rgba(0.13, 0.59, 0.95, 1.00);
 			break;
-		case BARCACHED:
-			cairo_pattern_add_color_stop_rgba(pat, 0, 1.00, 0.25, 0.90, 1);
-			cairo_pattern_add_color_stop_rgba(pat, 1, 0.75, 0.00, 0.65, 1);
+		case BARCACHED: // purple
+			pat = is_dark_theme(glab) ? cairo_pattern_create_rgba(0.00, 0.47, 0.71, 1.00) : cairo_pattern_create_rgba(0.61, 0.15, 0.69, 1.00);
 			break;
-		case BARFREE:
-			cairo_pattern_add_color_stop_rgba(pat, 0, 0.20, 1.00, 0.25, 1);
-			cairo_pattern_add_color_stop_rgba(pat, 1, 0.00, 0.75, 0.05, 1);
+		case BARFREE: // green
+			pat = is_dark_theme(glab) ? cairo_pattern_create_rgba(0.00, 0.59, 0.78, 1.00) : cairo_pattern_create_rgba(0.30, 0.69, 0.31, 1.00);
 			break;
-		case BARSWAP:
-			cairo_pattern_add_color_stop_rgba(pat, 0, 1.00, 0.35, 0.15, 1);
-			cairo_pattern_add_color_stop_rgba(pat, 1, 0.75, 0.15, 0.00, 1);
+		case BARSWAP: // red
+			pat = is_dark_theme(glab) ? cairo_pattern_create_rgba(0.00, 0.71, 0.85, 1.00) : cairo_pattern_create_rgba(0.96, 0.26, 0.21, 1.00);
 			break;
 	}
 
