@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2019 Xorg
+*    Copyright © 2014-2021 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -27,30 +27,22 @@
 #include <stdarg.h>
 #include <stdbool.h>
 #include <unistd.h>
+#include <ctype.h>
 #include <string.h>
 #include <signal.h>
 #include <execinfo.h>
 #include <getopt.h>
+#include <wordexp.h>
 #include <locale.h>
-#include <libintl.h>
 #include "cpu-x.h"
 
-#define HAS_WEB_SUPPORT (HAS_LIBCURL && HAS_LIBJSONC)
-#if HAS_WEB_SUPPORT
-# include <curl/curl.h>
-# include <json-c/json.h>
+#if HAS_GETTEXT
+# include <libintl.h>
 #endif
 
-#if PORTABLE_BINARY
-# include <sys/stat.h>
-# include <archive.h>
-# include <archive_entry.h>
-# if HAS_GETTEXT
-#  include "../po/mo.h"
-# endif
-#endif
 
-char *binary_name, *new_version[2] = { NULL, NULL };
+#define LOG_FILE "/tmp/cpu-x.log"
+
 Options *opts;
 
 
@@ -61,7 +53,7 @@ static void labels_setname(Labels *data)
 {
 	int i, j;
 
-	MSG_VERBOSE(_("Setting label names"));
+	MSG_VERBOSE("%s", _("Setting label names"));
 	/* CPU tab */
 	asprintf(&data->objects[TABCPU],                _("CPU")); // Tab label
 	asprintf(&data->objects[FRAMPROCESSOR],         _("Processor")); // Frame label
@@ -154,11 +146,15 @@ static void labels_setname(Labels *data)
 		j = i / GPUFIELDS;
 		asprintf(&data->objects[FRAMGPU1 + j],                   _("Card %i"), j); // Frame label
 		asprintf(&data->tab_graphics[NAME][GPU1VENDOR      + i], _("Vendor"));
+		asprintf(&data->tab_graphics[NAME][GPU1DRIVER      + i], _("Driver"));
 		asprintf(&data->tab_graphics[NAME][GPU1MODEL       + i], _("Model"));
 		asprintf(&data->tab_graphics[NAME][GPU1TEMPERATURE + i], _("Temperature"));
 		asprintf(&data->tab_graphics[NAME][GPU1USAGE       + i], _("Usage"));
+		asprintf(&data->tab_graphics[NAME][GPU1MEMUSED     + i], _("Memory Used"));
 		asprintf(&data->tab_graphics[NAME][GPU1CORECLOCK   + i], _("GPU clock"));
 		asprintf(&data->tab_graphics[NAME][GPU1MEMCLOCK    + i], _("Memory clock"));
+		asprintf(&data->tab_graphics[NAME][GPU1VOLTAGE     + i], _("Core Voltage"));
+		asprintf(&data->tab_graphics[NAME][GPU1POWERAVG    + i], _("Power Avg"));
 	}
 
 	/* Bench tab */
@@ -258,7 +254,7 @@ static void dump_data(Labels *data)
 		{ NO_GRAPHICS,    GPU4VENDOR,   FRAMGPU4            }
 	};
 
-	MSG_VERBOSE(_("Dumping data..."));
+	MSG_VERBOSE("%s", _("Dumping data…"));
 	for(i = 0; arrays[i].dim_names != NULL; i++)
 	{
 		MSG_STDOUT("  %s>>>>>>>>>> %s <<<<<<<<<<%s", col_start, data->objects[i], col_end);
@@ -275,7 +271,7 @@ static void dump_data(Labels *data)
 			}
 			MSG_STDOUT("%16s: %s", arrays[i].dim_names[j], arrays[i].dim_values[j]);
 		}
-		MSG_STDOUT("\n");
+		MSG_STDOUT("%s", "\n");
 	}
 
 	labels_free(data);
@@ -300,7 +296,7 @@ void labels_free(Labels *data)
 		{ NULL,                        NULL,                         0                               }
 	};
 
-	MSG_VERBOSE(_("Freeing memory"));
+	MSG_VERBOSE("%s", _("Freeing memory"));
 	for(i = 0; arrays[i].dim_names != NULL; i++)
 	{
 		for(j = 0; j < arrays[i].last; j++)
@@ -317,251 +313,6 @@ void labels_free(Labels *data)
 }
 
 
-/************************* Update-related functions *************************/
-
-#if HAS_WEB_SUPPORT
-/* Write function for Curl */
-static size_t writefunc(void *ptr, size_t size, size_t nmemb, void **stream)
-{
-	char **buff    = (char**) stream;
-	char *old_buff = *buff;
-	char *tmp      = NULL;
-	const size_t len     = size * nmemb;
-	const size_t old_len = (old_buff == NULL) ? 0 : strlen(old_buff);
-	const size_t new_len = old_len + len;
-
-	if((tmp = realloc(old_buff, new_len + 1)) == NULL)
-	{
-		MSG_ERRNO(_("could not reallocate memory"));
-		MSG_STDERR(_("Exiting %s"), PRGNAME);
-		exit(255);
-	}
-
-	*buff = tmp;
-	memcpy(&((*buff)[old_len]), ptr, len);
-	(*buff)[new_len] = '\0';
-
-	return len;
-}
-
-/* Check if running version is latest */
-static bool check_new_version(void)
-{
-	char *json = NULL;
-	CURL *curl;
-	CURLcode code;
-	json_object *jobj;
-
-	if(!opts->use_network)
-	{
-		asprintf(&new_version[1], "%c", '\0');
-		return false;
-	}
-
-	MSG_VERBOSE(_("Checking on Internet for a new version..."));
-	curl = curl_easy_init();
-	if(!curl)
-	{
-		MSG_ERROR(_("failed to open a Curl session"));
-		return 1;
-	}
-
-	curl_easy_setopt(curl, CURLOPT_URL, UPDURL);
-	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writefunc);
-	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 3L);
-	curl_easy_setopt(curl, CURLOPT_USERAGENT, "curl/" LIBCURL_VERSION);
-	curl_easy_setopt(curl, CURLOPT_WRITEDATA, &json);
-	code = curl_easy_perform(curl);
-	curl_easy_cleanup(curl);
-
-	if(code == CURLE_OK)
-	{
-		jobj = json_tokener_parse(json);
-		const char *tag_name = json_object_get_string(json_object_object_get(jobj, "tag_name"));
-		if((tag_name != NULL) && (strlen(tag_name) > 1))
-			asprintf(&new_version[0], &(tag_name[1])); // Remove the 'v' at beginning
-		json_object_put(jobj);
-	}
-	free(json);
-
-	if(new_version[0] == NULL)
-	{
-		MSG_ERROR(_("failed to perform the Curl transfer (%s)"),
-			(new_version[0] != NULL) ? curl_easy_strerror(code) : _("wrong write data"));
-		opts->use_network = false;
-		asprintf(&new_version[1], "%c", '\0');
-	}
-	else if(strcmp(new_version[0], PRGVER))
-	{
-		MSG_VERBOSE(_("A new version of %s is available!"), PRGNAME);
-		asprintf(&new_version[1], _("(version %s is available)"), new_version[0]);
-		return true;
-	}
-	else
-	{
-		MSG_VERBOSE(_("No new version available"));
-		asprintf(&new_version[1], _("(up-to-date)"));
-	}
-
-	free(new_version[0]);
-	new_version[0] = NULL;
-	return false;
-}
-
-# if 0 //PORTABLE_BINARY
-/* Copy function for libarchive */
-static int copy_data(struct archive *ar, struct archive *aw)
-{
-	int ret;
-	const void *buff;
-	size_t size;
-	la_int64_t offset;
-
-	while(true)
-	{
-		ret = archive_read_data_block(ar, &buff, &size, &offset);
-		if(ret == ARCHIVE_EOF)
-			return ARCHIVE_OK;
-		if(ret < ARCHIVE_OK)
-			return ret;
-
-		if((ret = archive_write_data_block(aw, buff, size, offset)) < ARCHIVE_OK)
-			return ret;
-	}
-}
-
-/* Extract a .tar.gz archive */
-static int extract_archive(const char *filename, const char *needed)
-{
-	int ret;
-	const int flags = ARCHIVE_EXTRACT_TIME | ARCHIVE_EXTRACT_PERM;
-	struct archive *archive, *ext, *archive_ptr;
-	struct archive_entry *entry;
-
-	archive = archive_read_new();
-	archive_read_support_format_tar(archive);
-	archive_read_support_filter_gzip(archive);
-
-	ext = archive_write_disk_new();
-	archive_write_disk_set_options(ext, flags);
-	archive_write_disk_set_standard_lookup(ext);
-
-	archive_ptr = archive;
-	if((ret = archive_read_open_filename(archive, filename, 10240)))
-		goto error;
-
-	do {
-		if((ret = archive_read_next_header(archive, &entry)) != ARCHIVE_OK)
-			goto error;
-	} while(strcmp(archive_entry_pathname(entry), needed));
-
-	archive_ptr = ext;
-	if((ret = archive_write_header(ext, entry)) != ARCHIVE_OK)
-		goto error;
-	if((ret = copy_data(archive, ext)) != ARCHIVE_OK)
-		goto error;
-	if((ret = archive_write_finish_entry(ext)) != ARCHIVE_OK)
-		goto error;
-
-	archive_read_close(archive);
-	archive_read_free(archive);
-	archive_write_close(ext);
-	archive_write_free(ext);
-	return 0;
-
-error:
-	MSG_ERROR(_("an error occurred while extracting %s archive (%s)"), filename, archive_error_string(archive_ptr));
-	return 1;
-}
-
-/* Apply new portable version if available */
-static int update_prg(void)
-{
-	int err;
-	char *archive = NULL, *new_binary = NULL;
-	CURL *curl;
-	CURLcode code;
-	FILE *file_descr = NULL;
-
-	if(!opts->use_network)
-	{
-		MSG_WARNING(_("Network access is disabled by environment variable"
-		              " (set CPUX_NETWORK with a positive value to enable it)"));
-		return 1;
-	}
-
-	if(new_version[0] == NULL)
-	{
-		MSG_WARNING(_("No new version available"));
-		return 2;
-	}
-
-	curl = curl_easy_init();
-	if(!curl)
-	{
-		MSG_ERROR(_("failed to open a Curl session"));
-		return 3;
-	}
-
-	asprintf(&archive, "%s_v%s_portable%s.tar.gz", PRGNAME, new_version[0], HAS_GTK ? "" : "_noGTK");
-	file_descr = fopen(archive, "wb");
-	if(file_descr == NULL)
-	{
-		MSG_ERRNO(_("failed to open %s archive for writing"), archive);
-		free(archive);
-		return 4;
-	}
-
-	/* Download archive */
-	MSG_VERBOSE(_("Downloading new version..."));
-	curl_easy_setopt(curl, CURLOPT_URL, format("%s/v%s/%s", TARBALL, new_version[0], archive));
-	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, NULL);
-	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
-	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 2 * 60L);
-	curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 3L);
-	curl_easy_setopt(curl, CURLOPT_WRITEDATA, file_descr);
-	code = curl_easy_perform(curl);
-	curl_easy_cleanup(curl);
-	fclose(file_descr);
-	if(code != CURLE_OK)
-	{
-		MSG_ERROR(_("failed to download %s archive (%s)"), archive, curl_easy_strerror(code));
-		free(archive);
-		return 5;
-	}
-
-	/* Extract archive */
-	MSG_VERBOSE(_("Extracting new version..."));
-	asprintf(&new_binary, "%s_v%s_portable%s.%s", PRGNAME, new_version[0], HAS_GTK ? "" : "_noGTK", OS);
-	err = extract_archive(archive, new_binary);
-	if(err)
-	{
-		remove(archive);
-		return err;
-	}
-
-	/* Rename new binary */
-	MSG_VERBOSE(_("Applying new version..."));
-	if(strstr(binary_name, PRGVER) != NULL) // If binary name contains version
-	{
-		err  = remove(binary_name); // Delete old version and keep new version
-		err += rename(new_binary, format("%s_v%s", PRGNAME, new_version[0]));
-	}
-	else
-		err = rename(new_binary, binary_name); // Erase old version by new version
-
-	err += remove(archive);
-	if(err)
-		MSG_ERROR(_("an error occurred while removing/renaming files"));
-	else
-		MSG_VERBOSE(_("Update successful!"));
-
-	return err;
-}
-# endif /* PORTABLE_BINARY */
-#endif /* HAS_WEB_SUPPORT */
-
-
 /************************* Options-related functions *************************/
 
 static const struct
@@ -573,22 +324,24 @@ static const struct
 	char       *description;
 } cpux_options[] =
 {
-	{ HAS_GTK,         'g', "gtk",       no_argument,       N_("Start graphical user interface (GUI) (default)")           },
-	{ HAS_NCURSES,     'n', "ncurses",   no_argument,       N_("Start text-based user interface (TUI)")                    },
-	{ true,            'd', "dump",      no_argument,       N_("Dump all data on standard output and exit")                },
-	{ HAS_DMIDECODE,   'D', "dmidecode", no_argument,       N_("Run embedded command dmidecode and exit")                  },
+	{ HAS_GTK,         'G', "gtk",       no_argument,       N_("Start graphical user interface (GUI) (default)")           },
+	{ HAS_NCURSES,     'N', "ncurses",   no_argument,       N_("Start text-based user interface (TUI)")                    },
+	{ true,            'D', "dump",      no_argument,       N_("Dump all data on standard output and exit")                },
+	{ HAS_DMIDECODE,   'M', "dmidecode", no_argument,       N_("Run embedded command dmidecode and exit")                  },
 	{ HAS_BANDWIDTH,   'B', "bandwidth", no_argument,       N_("Run embedded command bandwidth and exit")                  },
-	{ true,            'a', "tab",       required_argument, N_("Set default tab (integer)")                                },
-	{ HAS_LIBCPUID,    'c', "core",      required_argument, N_("Select CPU core to monitor (integer)")                     },
-	{ HAS_BANDWIDTH,   't', "cachetest", required_argument, N_("Set custom bandwidth test for CPU caches speed (integer)") },
 	{ true,            'r', "refresh",   required_argument, N_("Set custom time between two refreshes (in seconds)")       },
-	{ true,            'o', "nocolor",   no_argument,       N_("Disable colored output")                                   },
-	{ true,            'i', "issue-fmt", no_argument,       N_("Print required informations to paste in an issue")         },
+	{ true,            't', "tab",       required_argument, N_("Set default tab (integer)")                                },
+	{ HAS_LIBCPUID,    'c', "core",      required_argument, N_("Select CPU core to monitor (integer)")                     },
+	{ HAS_BANDWIDTH,   'b', "cachetest", required_argument, N_("Set custom bandwidth test for CPU caches speed (integer)") },
+	{ true,            'd', "daemon",    no_argument,       N_("Start and connect to daemon")                              },
 	{ true,            'v', "verbose",   no_argument,       N_("Verbose output")                                           },
-	{ PORTABLE_BINARY, 'u', "update",    no_argument,       N_("Update portable version if a new version is available")    },
 	{ true,            'h', "help",      no_argument,       N_("Print help and exit")                                      },
 	{ true,            'V', "version",   no_argument,       N_("Print version and exit")                                   },
-	{ true,            '0', NULL,        0,                 NULL                                                           }
+	{ true,              0, "nocolor",   no_argument,       N_("Disable colored output")                                   },
+	{ true,              0, "debug",     no_argument,       N_("Print information for debugging")                          },
+	{ true,              0, "issue-fmt", no_argument,       N_("Print required information to paste in an issue")          },
+	{ HAS_NCURSES,       0, "keymap",    required_argument, N_("Set key mapping for NCurses mode (a[rrows]|e[macs]|i[nverted-T]|v[im])") },
+	{ true,              0, NULL,        0,                 NULL                                                           }
 };
 
 static const struct
@@ -598,46 +351,69 @@ static const struct
 	char       *description;
 } cpux_env_vars[] =
 {
-	{ HAS_WEB_SUPPORT, "CPUX_NETWORK",        N_("Temporarily disable network support")                     },
-	{ true,            "CPUX_BCLK",           N_("Enforce the bus clock")                                   },
-	{ HAS_LIBCPUID,    "CPUX_CPUID_RAW",      N_("Read CPUID raw data from a given file")                   },
-	{ HAS_LIBCPUID,    "CPUX_DEBUG_DATABASE", N_("Only print a message if CPU does not belong in database") },
-	{ true,            NULL,                  NULL                                                          }
+	{ true,            "CPUX_ARGS",                N_("Add default command line arguments")                      },
+	{ true,            "CPUX_BCLK",                N_("Enforce the bus clock")                                   },
+	{ true,            "CPUX_FORCE_FREQ_FALLBACK", N_("Ignore CPU frequency reported by libcpuid") },
+	{ HAS_LIBCPUID,    "CPUX_CPUID_RAW",           N_("Read CPUID raw data from a given file")                   },
+	{ HAS_LIBCPUID,    "CPUX_DEBUG_DATABASE",      N_("Only print a message if CPU is not present in the database") },
+	{ true,            NULL,                       NULL                                                          }
 };
 
+/* Check for influenceable environment variables */
+static void check_environment_variables(Labels *data)
+{
+	if(getenv("CPUX_BCLK"))
+		data->bus_freq = atof(getenv("CPUX_BCLK"));
+	if(getenv("CPUX_FORCE_FREQ_FALLBACK"))
+		opts->freq_fallback = ((atoi(getenv("CPUX_FORCE_FREQ_FALLBACK"))) > 0);
+	if(getenv("CPUX_CPUID_RAW"))
+		data->l_data->cpuid_raw_file = getenv("CPUX_CPUID_RAW");
+	if(getenv("CPUX_DEBUG_DATABASE"))
+		opts->debug_database = ((atoi(getenv("CPUX_DEBUG_DATABASE"))) > 0);
+}
+
 /* This is help display with --help option */
-static void help(void)
+static void help(char *binary_name)
 {
 	int i;
+	bool options_header = false;
+	char buff[MAXSTR];
 
 	MSG_STDOUT(_("Usage: %s DISPLAY [OPTIONS]\n"), binary_name);
-	MSG_STDOUT(_("Available DISPLAY:"));
+	MSG_STDOUT("%s", _("Available DISPLAY:"));
 	for(i = 0; cpux_options[i].long_opt != NULL; i++)
 	{
-		if(cpux_options[i].short_opt == 'a')
-			MSG_STDOUT(_("\nAvailable OPTIONS:"));
-		if(cpux_options[i].has_mod)
-			MSG_STDOUT("  -%c, --%-10s %s", cpux_options[i].short_opt, cpux_options[i].long_opt, _(cpux_options[i].description));
+		if(!cpux_options[i].has_mod)
+			continue;
+
+		if(!options_header && islower(cpux_options[i].short_opt))
+		{
+			options_header = true;
+			MSG_STDOUT("\n%s", _("Available OPTIONS:"));
+		}
+
+		if(cpux_options[i].short_opt) snprintf(buff, MAXSTR, "  -%c,", cpux_options[i].short_opt);
+		else                          snprintf(buff, MAXSTR, "     ");
+		MSG_STDOUT("%s --%-10s %s", buff, cpux_options[i].long_opt, _(cpux_options[i].description));
 	}
 
-	MSG_STDOUT(_("\nInfluenceable environment variables:"));
+	MSG_STDOUT("\n%s", _("Influenceable environment variables:"));
 	for(i = 0; cpux_env_vars[i].var_name != NULL; i++)
 	{
-		if(cpux_env_vars[i].has_mod)
-			MSG_STDOUT("  %-20s %s", cpux_env_vars[i].var_name, _(cpux_env_vars[i].description));
+		if(!cpux_env_vars[i].has_mod)
+			continue;
+		MSG_STDOUT("  %-25s %s", cpux_env_vars[i].var_name, _(cpux_env_vars[i].description));
 	}
 }
 
 /* This is the --version option */
-static void version(void)
+static void version(bool full_header)
 {
 	int i;
 	const struct { const bool has_mod; const char *lib, *version; } libs_ver[] =
 	{
 		{ HAS_GTK,         "GTK",         GTK_VERSION         },
 		{ HAS_NCURSES,     "NCURSES",     NCURSES_VERSION     },
-		{ HAS_LIBCURL,     "LIBCURL",     LIBCURL_VERSION     },
-		{ HAS_LIBJSONC,    "LIBJSONC",    LIBJSONC_VERSION    },
 		{ HAS_LIBCPUID,    "LIBCPUID",    LIBCPUID_VERSION    },
 		{ HAS_LIBPCI,      "LIBPCI",      LIBPCI_VERSION      },
 		{ HAS_LIBPROCPS,   "LIBPROCPS",   LIBPROCPS_VERSION   },
@@ -647,16 +423,14 @@ static void version(void)
 		{ false,           NULL,          NULL                }
 	};
 
-	if(HAS_WEB_SUPPORT)
-		check_new_version();
-
-	MSG_STDOUT("%s %s %s", PRGNAME, PRGVER, new_version[1]);
-	MSG_STDOUT("%s\n", PRGCPRGHT);
-	MSG_STDOUT(_("This is free software: you are free to change and redistribute it."));
-	MSG_STDOUT(_("This program comes with ABSOLUTELY NO WARRANTY"));
-	MSG_STDOUT(_("See the %s license: <%s>\n"), PRGLCNS, LCNSURL);
-	MSG_STDOUT(_("Built on %s, %s (with %s %s on %s)."), __DATE__, __TIME__, CC, __VERSION__, OS);
-	free(new_version[1]);
+	PRGINFO(stdout);
+	if(full_header)
+	{
+		MSG_STDOUT("%s\n", PRGCPRGHT);
+		MSG_STDOUT("%s", _("This is free software: you are free to change and redistribute it."));
+		MSG_STDOUT("%s", _("This program comes with ABSOLUTELY NO WARRANTY"));
+		MSG_STDOUT(_("See the %s license: <%s>\n"), PRGLCNS, LCNSURL);
+	}
 
 	/* Print features version */
 	for(i = 0; libs_ver[i].lib != NULL; i++)
@@ -666,27 +440,70 @@ static void version(void)
 	}
 }
 
-/* Parse arguments and set some flags */
-#define OPTIONS_COUNT (sizeof(cpux_options) / sizeof(cpux_options[0]) - 1)
-static void parse_arguments(int argc, char *argv[])
+/* Add arguments from environment variable CPUX_ARGS */
+static void environment_to_arguments(int *argc, char ***argv)
 {
-	int i, j = 0, c, tmp_arg = -1;
-	char shortopts[OPTIONS_COUNT * 2] = "";
+	int err = 0;
+	const char *args = getenv("CPUX_ARGS");
+	wordexp_t we;
+
+	if(args == NULL)
+		return;
+
+	if((err = wordexp(args, &we, 0)) != 0)
+	{
+		MSG_ERROR(_("failed to call wordexp (%i)"), err);
+		return;
+	}
+
+	*argc = we.we_wordc;
+	*argv = malloc((we.we_wordc + 1) * sizeof(char *));
+	for(size_t i = 0; i < we.we_wordc; i++)
+	{
+		const size_t length = strlen(we.we_wordv[i]) + 1;
+		(*argv)[i + 1] = malloc(length * sizeof(char));
+		memcpy((*argv)[i + 1], we.we_wordv[i], length);
+	}
+	wordfree(&we);
+}
+
+/* Parse arguments and set some flags */
+#define OPTIONS_COUNT   (sizeof(cpux_options) / sizeof(cpux_options[0]))
+#define SHORT_OPT_SIZE  3
+#define SHORT_OPTS_SIZE (OPTIONS_COUNT * 2)
+static void parse_arguments(int argc_orig, char *argv_orig[])
+{
+	int i, j = 0, c, argc = 0, longindex, tmp_arg = -1;
+	char shortopt[SHORT_OPT_SIZE], shortopts[SHORT_OPTS_SIZE] = "";
+	char **argv = NULL, **tmp = NULL;
 	struct option longopts[OPTIONS_COUNT];
 
-	/* Filling longopts structure */
+	/* Inject arguments from environment */
+	environment_to_arguments(&argc, &argv);
+	c     = argc;
+	argc += argc_orig;
+	tmp   = realloc(argv, argc * sizeof(char *));
+	ALLOC_CHECK(tmp);
+	argv  = tmp;
+	for(i = 0; i < argc_orig; i++)
+	{
+		j = (i == 0) ? 0 : c + i; // Preserve argv[0], then append original argv at end of new array
+		const size_t length = strlen(argv_orig[i]) + 1;
+		argv[j] = malloc(length * sizeof(char));
+		memcpy(argv[j], argv_orig[i], length);
+	}
+
+	/* Fill longopts structure */
+	j = 0;
 	for(i = 0; cpux_options[i].long_opt != NULL; i++)
 	{
 		while(!cpux_options[i].has_mod)
 			i++;
 		longopts[j++] = (struct option) { .name = cpux_options[i].long_opt, .has_arg = cpux_options[i].need_arg, .flag = 0, .val = cpux_options[i].short_opt };
-		strcat(shortopts, format("%c%s", cpux_options[i].short_opt, cpux_options[i].need_arg ? ":" : ""));
+		snprintf(shortopt, SHORT_OPT_SIZE, "%c%c", cpux_options[i].short_opt, cpux_options[i].need_arg ? ':' : '\0');
+		strncat(shortopts, shortopt, SHORT_OPTS_SIZE - 1);
 	}
-
-	/* Avoid uninitialized members */
-	c = OPTIONS_COUNT - 1;
-	for(i = j; i < (int) OPTIONS_COUNT; i++)
-		longopts[i] = (struct option) { .name = cpux_options[c].long_opt, .has_arg = cpux_options[c].need_arg, .flag = 0, .val = cpux_options[c].short_opt };
+	longopts[j] = (struct option) { 0, 0, 0, 0 };
 
 	/* Set the default mode */
 	if(HAS_GTK && (getenv("DISPLAY") != NULL || getenv("WAYLAND_DISPLAY") != NULL))
@@ -697,25 +514,31 @@ static void parse_arguments(int argc, char *argv[])
 		opts->output_type = OUT_DUMP;
 
 	/* Parse options */
-	while((c = getopt_long(argc, argv, shortopts, longopts, NULL)) != -1)
+	while((c = getopt_long(argc, argv, shortopts, longopts, &longindex)) != -1)
 	{
 		switch(c)
 		{
-			case 'g':
+			case 'G':
 				opts->output_type = OUT_GTK;
 				break;
-			case 'n':
+			case 'N':
 				opts->output_type = OUT_NCURSES;
 				break;
-			case 'd':
+			case 'D':
 				opts->output_type = OUT_DUMP;
 				break;
+			case 'M':
+				opts->output_type = OUT_DMIDECODE;
+				break;
+			case 'B':
+				opts->output_type = OUT_BANDWIDTH;
+				break;
 			case 'r':
 				tmp_arg = atoi(optarg);
 				if(tmp_arg >= 1)
 					opts->refr_time = tmp_arg;
 				break;
-			case 'a':
+			case 't':
 				tmp_arg = atoi(optarg);
 				if(NO_CPU < tmp_arg && tmp_arg <= NO_ABOUT)
 					opts->selected_page = tmp_arg;
@@ -725,63 +548,80 @@ static void parse_arguments(int argc, char *argv[])
 				if(tmp_arg >= 0)
 					opts->selected_core = tmp_arg;
 				break;
-			case 't':
+			case 'b':
 				tmp_arg = atoi(optarg);
 				if(tmp_arg >= 0)
 					opts->bw_test = atoi(optarg);
 				break;
-			case 'D':
-				opts->output_type = OUT_DMIDECODE;
-				break;
-			case 'B':
-				opts->output_type = OUT_BANDWIDTH;
-				break;
-			case 'o':
-				opts->color = false;
-				break;
-			case 'i':
-				opts->color       = false;
-				opts->verbose     = true;
-				opts->issue       = true;
-				opts->use_network = 0;
-				opts->output_type = OUT_DUMP;
-				setlocale(LC_ALL, "C");
-				version();
+			case 'd':
+				opts->with_daemon = true;
 				break;
 			case 'v':
 				opts->verbose = true;
 				break;
-			case 'u':
-				opts->update = true;
-				break;
 			case 'h':
-				help();
+				help(argv[0]);
 				exit(EXIT_SUCCESS);
+				break;
 			case 'V':
-				version();
+				version(true);
 				exit(EXIT_SUCCESS);
+				break;
+			case 0:
+				if(!strcmp(longopts[longindex].name, "nocolor"))
+				{
+					opts->color = false;
+					break;
+				}
+				else if(!strcmp(longopts[longindex].name, "debug"))
+				{
+					opts->debug = true;
+					break;
+				}
+				else if(!strcmp(longopts[longindex].name, "issue-fmt"))
+				{
+					opts->color       = false;
+					opts->verbose     = true;
+					opts->debug       = true;
+					opts->issue       = true;
+					opts->use_network = 0;
+					opts->output_type = OUT_DUMP;
+					setlocale(LC_ALL, "C");
+					setenv("CPUX_DAEMON_DEBUG", "1", 1);
+					unlink(LOG_FILE);
+					freopen(LOG_FILE, "a", stdout);
+					setvbuf(stdout, NULL, _IONBF, 0);
+					dup2(STDOUT_FILENO, STDERR_FILENO);
+					version(false);
+					break;
+				}
+				else if(!strcmp(longopts[longindex].name, "keymap"))
+				{
+					switch(optarg[0])
+					{
+						case 'a': opts->keymap = ARROWS;     break;
+						case 'e': opts->keymap = EMACS;      break;
+						case 'i': opts->keymap = INVERTED_T; break;
+						case 'v': opts->keymap = VIM;        break;
+						default: help(argv[0]); exit(EXIT_FAILURE);
+					}
+					break;
+				}
+				/* Fall through */
 			case '?':
 			default:
-				help();
+				help(argv[0]);
 				exit(EXIT_FAILURE);
 		}
 	}
-}
 
-/* Check for influenceable environment variables */
-static void check_environment_variables(Labels *data)
-{
-	if(getenv("CPUX_NETWORK"))
-		opts->use_network = ((atoi(getenv("CPUX_NETWORK"))) > 0);
-	if(getenv("CPUX_BCLK"))
-		data->bus_freq = atof(getenv("CPUX_BCLK"));
-	if(getenv("CPUX_CPUID_RAW"))
-		data->l_data->cpuid_raw_file = getenv("CPUX_CPUID_RAW");
-	if(getenv("CPUX_DEBUG_DATABASE"))
-		opts->debug_database = ((atoi(getenv("CPUX_DEBUG_DATABASE"))) > 0);
-	if(getenv("CPUX_FORCE_FREQ_FALLBACK"))
-		opts->freq_fallback = ((atoi(getenv("CPUX_FORCE_FREQ_FALLBACK"))) > 0);
+	for(i = 0; i < argc; i++)
+		free(argv[i]);
+	free(argv);
 }
+#undef OPTIONS_COUNT
+#undef SHORT_OPT_SIZE
+#undef SHORT_OPTS_SIZE
 
 
 /************************* Main-related functions *************************/
@@ -800,8 +640,8 @@ static void sighandler(int signum)
 	/* Print the backtrace */
 	MSG_STDERR(_("\n%sOops, something was wrong! %s has received signal %d (%s) and has crashed.%s"),
 	           BOLD_RED, PRGNAME, signum, strsignal(signum), DEFAULT);
-	MSG_STDERR("========================= Backtrace =========================");
-	MSG_STDERR("%s %s (%s, %s)", PRGNAME, PRGVER, CC, OS);
+	MSG_STDERR("%s", "========================= Backtrace =========================");
+	PRGINFO(stderr);
 	for(i = 1; i < bt_size; i++)
 	{
 		char *address = strtok(strrchr(strdup(bt_syms[i]), '[') + 1, "]");
@@ -812,9 +652,9 @@ static void sighandler(int signum)
 			MSG_STDERR("#%2i %s", i, bt_syms[i]);
 		free(buff);
 	}
-	MSG_STDERR("======================== End Backtrace =======================\n");
-	MSG_STDERR(_("You can paste this backtrace by opening a new issue here:"));
-	MSG_STDERR("https://github.com/X0rg/CPU-X/issues/new\n");
+	MSG_STDERR("%s", "======================== End Backtrace =======================\n");
+	MSG_STDERR("%s", _("You can paste this backtrace by opening a new issue here:"));
+	MSG_STDERR("%s", "https://github.com/X0rg/CPU-X/issues/new\n");
 
 	/* Stop program */
 	free(bt_syms);
@@ -823,6 +663,7 @@ static void sighandler(int signum)
 }
 
  /* Enable internationalization support */
+#if HAS_GETTEXT
 static int set_locales(void)
 {
 	int err;
@@ -831,38 +672,6 @@ static int set_locales(void)
 	if(TEXTDOMAINDIR == NULL || TEXTDOMAINDIR[0] == '\0')
 		TEXTDOMAINDIR = LOCALEDIR;
 
-#if PORTABLE_BINARY && HAS_GETTEXT
-	int i;
-	FILE *mofile = NULL;
-
-	if(!access(LOCALEDIR, R_OK))
-		goto end_extraction;
-
-	/* Write .mo files in temporary directory */
-	err = mkdir(LOCALEDIR, ACCESSPERMS);
-	for(i = 0; ptrlen[i] != NULL; i++)
-	{
-		err   += mkdir(format("%s/%s/",             LOCALEDIR, lang[i]), ACCESSPERMS);
-		err   += mkdir(format("%s/%s/LC_MESSAGES/", LOCALEDIR, lang[i]), ACCESSPERMS);
-		mofile = fopen(format("%s/%s/LC_MESSAGES/%s.mo", LOCALEDIR, lang[i], GETTEXT_PACKAGE), "w");
-
-		if(mofile != NULL)
-		{
-			err += fwrite(ptrlang[i], sizeof(unsigned char), *(ptrlen)[i], mofile) > 0 ? 0 : 1;
-			err += fclose(mofile);
-		}
-		else
-			err++;
-	}
-
-	/* Override TEXTDOMAINDIR in portable binary */
-	TEXTDOMAINDIR = LOCALEDIR;
-
-	if(err)
-		MSG_ERROR("an error occurred while extracting translations");
-end_extraction:
-#endif /* PORTABLE_BINARY && HAS_GETTEXT */
-
 	/* Apply locale */
 	setlocale(LC_ALL, "");
 	err  = bindtextdomain(GETTEXT_PACKAGE, TEXTDOMAINDIR)    ? 0 : 1;
@@ -872,17 +681,17 @@ end_extraction:
 	/* Check if something is wrong */
 	if(err)
 	{
-		MSG_ERROR(_("an error occurred while setting locale"));
+		MSG_ERROR("%s", _("an error occurred while setting locale"));
 		return 1;
 	}
 	else
 		return 0;
 }
+#endif /* HAS_GETTEXT */
 
 int main(int argc, char *argv[])
 {
 	/* Init variables */
-	binary_name  = argv[0];
 	Labels *data = &(Labels)
 	{
 		.tab_cpu         = { { NULL } },
@@ -893,13 +702,15 @@ int main(int argc, char *argv[])
 		.tab_graphics    = { { NULL } },
 		.tab_bench       = { { NULL } },
 		.cpu_freq        = 0,
+		.socket_fd       = -1,
 		.cpu_count       = 0,
 		.cache_count     = 0,
 		.dimm_count      = 0,
 		.gpu_count       = 0,
 		.bus_freq        = 0.0,
 		.cpu_min_mult    = 0.0,
-		.cpu_max_mult    = 0.0
+		.cpu_max_mult    = 0.0,
+		.reload          = false
 	};
 	data->l_data = &(LibcpuidData)
 	{
@@ -941,40 +752,45 @@ int main(int argc, char *argv[])
 	{
 		.color          = true,
 		.verbose        = false,
+		.debug          = false,
 		.issue          = false,
 		.use_network    = true,
-		.update         = false,
+		.with_daemon    = false,
 		.debug_database = false,
 		.freq_fallback  = false,
 		.selected_page  = 0,
 		.selected_core  = 0,
 		.bw_test        = 0,
-		.refr_time      = 1
+		.refr_time      = 1,
+		.keymap         = ARROWS
 	};
 
+#if HAS_GETTEXT
 	set_locales();
+#endif /* HAS_GETTEXT */
 	signal(SIGSEGV, sighandler);
 	signal(SIGFPE,  sighandler);
 	signal(SIGABRT, sighandler);
 
 	/* Parse options */
+#if HAS_GTK
+	load_settings();
+#endif /* HAS_GTK */
 	parse_arguments(argc, argv);
 	check_environment_variables(data);
 	if(opts->output_type > OUT_NO_CPUX)
 		goto skip_init;
 
+	/* Connect to daemon */
+	if(IS_ROOT || opts->with_daemon)
+		start_daemon(false);
+	if(daemon_is_alive())
+		connect_to_daemon(data);
+
 	/* Retrieve data */
-	if(getuid())
-	{
-		MSG_WARNING(_("Root privileges are required to work properly"));
-		MSG_WARNING(_("Some informations will not be retrievable"));
-	}
 	labels_setname(data);
 	fill_labels   (data);
 
-	if(HAS_WEB_SUPPORT)
-		check_new_version();
-
 	/* Show data */
 	if(HAS_GTK && (opts->output_type == OUT_GTK))
 		start_gui_gtk(&argc, &argv, data);
@@ -988,16 +804,5 @@ skip_init:
 	if(HAS_BANDWIDTH && (opts->output_type == OUT_BANDWIDTH))
 		return run_bandwidth();
 
-#if 0 //PORTABLE_BINARY
-	/* Only 64-bit portable binary can be updated since v3.2.1 */
-	if(PORTABLE_BINARY && HAS_WEB_SUPPORT && opts->update) {
-# ifdef __x86_64__
-		update_prg();
-# else
-		MSG_ERROR(_("Sorry, you cannot update %s: 32-bit portable version is no more supported."), PRGNAME);
-# endif
-	}
-#endif /* PORTABLE_BINARY */
-
 	return EXIT_SUCCESS;
 }
