@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2020 Xorg
+*    Copyright © 2014-2021 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -118,7 +118,7 @@ void start_tui_ncurses(Labels *data)
 
 	while(ch != 'q')
 	{
-		ch = getch();
+		ch = convert_char(getch());
 		switch(ch)
 		{
 			case KEY_LEFT:
@@ -249,13 +249,51 @@ void start_tui_ncurses(Labels *data)
 
 /************************* Private functions *************************/
 
+/* Convert keys when an alternative mapping is used */
+#define ALT_CODE 27
+#define ALT(x)   (x & ALT_CODE)
+static int convert_char(int ch)
+{
+	int i = 0;
+	static int modifier = 0;
+	const int ch_mod = modifier ? ch & modifier : ch;
+
+	const int keymaps[][LASTKEYMAP] =
+	{
+		//Arrow       Emacs       Inverted-T  Vim
+		{ KEY_LEFT,   CTRL('b'),  'j',        'h'          },
+		{ KEY_RIGHT,  CTRL('f'),  'l',        'l'          },
+		{ KEY_UP,     CTRL('p'),  'i',        'k'          },
+		{ KEY_DOWN,   CTRL('n'),  'k',        'j'          },
+		{ KEY_PPAGE,  CTRL('v'),  CTRL('b'),  CTRL('b')    },
+		{ KEY_NPAGE,  ALT('v'),   CTRL('f'),  CTRL('f')    },
+		{ 'h',        '?',        '?',        '?'          },
+		{ ch_mod,     ch_mod,     ch_mod,     ch_mod       } // Sentinel value
+	};
+
+	modifier = (ch == ALT_CODE) ? ch : 0;
+	while(keymaps[i][opts->keymap] != ch_mod)
+		i++;
+
+	return keymaps[i][0];
+}
+#undef ALT_CODE
+#undef ALT
+
 /* Put window in the center of the screen */
 static bool resize_window(WINDOW *pwin, const SizeInfo info)
 {
+	static int prev_lines = -1, prev_cols = -1;
+
 	bool ret = true;
 	const int starty = (LINES - info.height) / 2;
 	const int startx = (COLS  - info.width)  / 2;
 
+	if (prev_lines >= 0 && prev_lines == LINES &&
+	    prev_cols >= 0 && prev_cols == COLS)
+		return true;
+	prev_lines = LINES;
+	prev_cols = COLS;
 	erase();
 	if((startx < 0) || (starty < 0))
 	{
@@ -389,8 +427,13 @@ static void nrefresh(NThrd *refr)
 				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1TEMPERATURE + i], data->tab_graphics[VALUE][GPU1TEMPERATURE + i]);
 				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1USAGE       + i], data->tab_graphics[VALUE][GPU1USAGE       + i]);
 				line++;
+				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1MEMUSED     + i], data->tab_graphics[VALUE][GPU1MEMUSED     + i]);
+				line++;
 				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1CORECLOCK + i], data->tab_graphics[VALUE][GPU1CORECLOCK     + i]);
 				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1MEMCLOCK  + i], data->tab_graphics[VALUE][GPU1MEMCLOCK      + i]);
+				line++;
+				mvwprintw2c(win, line, info.tb, "%13s: %s", data->tab_graphics[NAME][GPU1VOLTAGE   + i], data->tab_graphics[VALUE][GPU1VOLTAGE       + i]);
+				mvwprintw2c(win, line, info.tm, "%18s: %s", data->tab_graphics[NAME][GPU1POWERAVG  + i], data->tab_graphics[VALUE][GPU1POWERAVG      + i]);
 				line += LINE_3 - 1;
 			}
 			break;
@@ -714,7 +757,7 @@ static void draw_bar(WINDOW *win, const SizeInfo info, Labels *data, int bar)
 /* Graphics tab */
 static void ntab_graphics(WINDOW *win, const SizeInfo info, Labels *data)
 {
-	int i, line, start = LINE_0, end = LINE_5;
+	int i, line, start = LINE_0, end = LINE_7;
 
 	if(!data->gpu_count)
 		return;
@@ -746,6 +789,10 @@ static void ntab_graphics(WINDOW *win, const SizeInfo info, Labels *data)
 			case GPU2MEMCLOCK:
 			case GPU3MEMCLOCK:
 			case GPU4MEMCLOCK:
+			case GPU1POWERAVG:
+			case GPU2POWERAVG:
+			case GPU3POWERAVG:
+			case GPU4POWERAVG:
 				mvwprintw2c(win, line - 1, info.tm, "%18s: %s", data->tab_graphics[NAME][i], data->tab_graphics[VALUE][i]);
 				break;
 			default:
