@@ -1,5 +1,5 @@
 /****************************************************************************
-*    Copyright © 2014-2020 Xorg
+*    Copyright © 2014-2021 Xorg
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
@@ -97,6 +97,9 @@ typedef struct
 
 /********************************** TUI  **********************************/
 
+/* Convert keys when an alternative mapping is used */
+static int convert_char(int ch);
+
 /* Put window in the center of the screen */
 static bool resize_window(WINDOW *pwin, const SizeInfo info);
 
